@@ -1,5 +1,5 @@
 package Mojo::Weixin;
-our $VERSION = '1.2.3';
+our $VERSION = '1.2.4';
 use Mojo::Weixin::Base 'Mojo::EventEmitter';
 use Mojo::IOLoop;
 use Mojo::Weixin::Log;
@@ -203,7 +203,7 @@ sub new {
     $self->check_pid();
     $self->save_state();
     $SIG{CHLD} = 'IGNORE';
-    $SIG{INT} = $SIG{KILL} = $SIG{TERM} = $SIG{HUP} = sub{
+    $SIG{INT}  = $SIG{TERM} = $SIG{HUP} = sub{
         return if $^O ne 'MSWin32' and $_[0] eq 'INT' and !-t;
         $self->info("捕获到停止信号[$_[0]]，准备停止...");
         $self->clean_qrcode();
