@@ -6,11 +6,15 @@ sub Mojo::Weixin::_webwxgetheadimg {
     my $self = shift;
     my $object = shift;
     my $callback = shift;
+    if(not $object->_avatar){
+        $self->warn("对象[ " . $object->displayname . " ]头像地址无效");
+        return;
+    }
     my $url = 'https://' . $self->domain . $object->_avatar;
     $self->http_get($url,{Referer=>'https://' . $self->domain .'/'},sub{
         my ($data,$ua,$tx) = @_;
         if(not defined $data){
-            $self->debug("获取对象[ " . $object->displayname . " ]头像失败");
+            $self->warn("获取对象[ " . $object->displayname . " ]头像失败: 数据无效");
             return;
         }
         my $mime =  $data=~/^GIF8/          ?   'image/gif'
@@ -22,16 +26,12 @@ sub Mojo::Weixin::_webwxgetheadimg {
                     :   $mime=~/^image\/png/i          ?   ".png"
                     :   $mime=~/^image\/bmp/i          ?   ".bmp"
                     :   $mime=~/^image\/gif/i          ?   ".gif"
-                    :   $mime=~/^text\/plain/i         ?   ".txt"
-                    :   $mime=~/^text\/html/i          ?   ".html"
-                    :   $mime=~/^text\/json/i          ?   ".json"
-                    :   $mime=~/^application\/json/i   ?   ".json"
-                    :   $mime=~/^video\/mp4/i          ?   ".mp4"
-                    :   $mime=~/^audio\/mp3/i          ?   ".mp3"
-                    :   $mime=~/^application\/json/i   ?   ".json"
-                    :                                      ".dat"
+                    :                                      undef
         ; 
-        return unless defined $type;
+        if(not defined $type){
+            $self->warn("获取对象[ " . $object->displayname . " ]头像失败: 非图像类型数据");
+            return;
+        }
         if(defined $self->media_dir and not -d $self->media_dir){
             $self->error("无效的 media_dir: " . $self->media_dir);
             return;
